@@ -19,7 +19,8 @@ else
 end
 
 PCS_BIN = "#{prefix_path}pcs" unless defined? PCS_BIN
-CRMDIFF_BIN = "#{prefix_path}crm_diff unless" unless defined? CRMDIFF_BIN
+CRMDIFF_BIN = "#{prefix_path}crm_diff" unless defined? CRMDIFF_BIN
+CRMNODE_BIN = "#{prefix_path}crm_node" unless defined? CRMNODE_BIN
 CRMSIMULATE_BIN = "#{prefix_path}crm_simulate" unless defined? CRMSIMULATE_BIN
 CRMRESOURCE_BIN = "#{prefix_path}crm_resource" unless defined? CRMRESOURCE_BIN
 TIMEOUT_BIN = "#{prefix_path}timeout" unless defined? TIMEOUT_BIN
@@ -36,6 +37,26 @@ def pcs_cli_version()
   return pcs_cli_version
 end
 
+# returns 'show' or 'config' depending on the pcs version
+# In case pcs returns '' we choose the more recent 'config' default
+# (Although in that case it is likely it will fail differently anyways)
+def pcs_config_or_show()
+  if Puppet::Util::Package.versioncmp(pcs_cli_version(), '0.10.0') < 0
+    return 'show'
+  else
+    return 'config'
+  end
+end
+
+def crm_node_l()
+  begin
+    nodes = `#{CRMNODE_BIN} -l`
+  rescue
+    nodes = ''
+  end
+  return nodes
+end
+
 
 # Ruby 2.5 has dropped Dir::Tmpname.make_tmpname
 # https://github.com/ruby/ruby/commit/25d56ea7b7b52dc81af30c92a9a0e2d2dab6ff27
@@ -113,29 +134,38 @@ def pcs(name, resource_name, cmd, tries=1, try_sleep=0,
   if name.start_with?("create") && verify_on_create
     return pcs_create_with_verify(name, resource_name, cmd, tries, try_sleep)
   end
-  max_tries = name.include?('show') ? 1 : tries
+  max_tries = tries
   max_tries.times do |try|
-    cib = backup_cib()
-    try_text = max_tries > 1 ? "try #{try+1}/#{max_tries}: " : ''
-    Puppet.debug("#{try_text}#{PCS_BIN} -f #{cib} #{cmd}")
-    pcs_out = `#{PCS_BIN} -f #{cib} #{cmd} 2>&1`
-    if name.include?('show')
-      delete_cib(cib)
-      # return output for good exit or false for failure.
-      return $?.exitstatus == 0 ? pcs_out : false
-    end
-    if $?.exitstatus == 0
-      # If push_cib failed, we stay in the loop and keep trying
-      if push_cib(cib) == 0
-        sleep post_success_sleep
-        return pcs_out
+    begin
+      try_text = max_tries > 1 ? "try #{try+1}/#{max_tries}: " : ''
+      cib = backup_cib()
+      Puppet.debug("#{try_text}#{PCS_BIN} -f #{cib} #{cmd}")
+      pcs_out = `#{PCS_BIN} -f #{cib} #{cmd} 2>&1`
+      if name.include?('show')
+        delete_cib(cib)
+        # return output for good exit or false for failure.
+        return $?.exitstatus == 0 ? pcs_out : false
       end
+      if $?.exitstatus == 0
+        # If push_cib failed, we stay in the loop and keep trying
+        if push_cib(cib) == 0
+          sleep post_success_sleep
+          return pcs_out
+        end
+      end
+      Puppet.debug("Error: #{pcs_out}")
+    rescue Puppet::Error
+      Puppet.debug("cib_backup failed. Retrying #{try_text}")
     end
-    Puppet.debug("Error: #{pcs_out}")
     if try == max_tries-1
-      delete_cib(cib)
-      pcs_out_line = pcs_out.lines.first ? pcs_out.lines.first.chomp! : ''
-      raise Puppet::Error, "pcs -f #{cib} #{name} failed: #{pcs_out_line}"
+      # need to consider the case that pcs_out was always nil due to cib_backup() always failing
+      delete_cib(cib) if cib
+      if pcs_out == nil
+        pcs_out_line = ''
+      else
+        pcs_out_line = pcs_out.lines.first ? pcs_out.lines.first.chomp! : ''
+      end
+      raise Puppet::Error, "pcs -f #{cib} #{cmd} failed: #{pcs_out_line}. Too many tries"
     end
     if try_sleep > 0
       Puppet.debug("Sleeping for #{try_sleep} seconds between tries")
@@ -172,7 +202,7 @@ def pcs_create_with_verify(name, resource_name, cmd, tries=1, try_sleep=0)
     pcs_out = `#{PCS_BIN} #{cmd} 2>&1`
     if $?.exitstatus == 0
       sleep try_sleep
-      cmd_show = "#{PCS_BIN} resource show " + resource_name
+      cmd_show = "#{PCS_BIN} resource " + pcs_config_or_show() + " " + resource_name
       Puppet.debug("Verifying with: "+cmd_show)
       `#{cmd_show}`
       if $?.exitstatus == 0
